#!perl -T

use Test::More;
eval "use Test::Pod 1.14";
plan skip_all => "Author test" unless $ENV{AUTHOR_TESTING};
plan skip_all => "Test::Pod 1.14 required for testing POD: $@" if $@;
all_pod_files_ok();
